--------------------------------------------------------------------------------
-- Copyright (c) 2001 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2S Application Note: SoC with XR16 RISC CPU.
-- File:        Core.vhd
-- Description: Core Logic
-- History:     2001oct05, FB, Created.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity Core is
	port(
		clk48 : in  STD_LOGIC;
		clk25 : in  STD_LOGIC;
		rst   : in  STD_LOGIC;
		-- push buttons, switches, leds, 7-segment
		pb    : in  STD_LOGIC_VECTOR (7 downto 0);
		sw    : in  STD_LOGIC_VECTOR (7 downto 0);
		disp  : out STD_LOGIC_VECTOR (15 downto 0);
		led   : out STD_LOGIC_VECTOR (7 downto 0);
		-- usb
		rxd   : in  STD_LOGIC;
		rx0   : in  STD_LOGIC;
		txd   : out STD_LOGIC;
		tx0   : out STD_LOGIC;
		txoe  : out STD_LOGIC;
		-- vga
		r     : out STD_LOGIC;
		g     : out STD_LOGIC;
		b     : out STD_LOGIC;
		i     : out STD_LOGIC;
		row   : in  STD_LOGIC_VECTOR (9 downto 0);
		col   : in  STD_LOGIC_VECTOR (9 downto 0);
		hblk  : in  STD_LOGIC;
		vblk  : in  STD_LOGIC
		);
end Core;

architecture BHV of Core is
	
	---- Component declarations -----
	
	component Glue
		port (
			adx_nxt : in STD_LOGIC_VECTOR (15 downto 0);
			clk : in STD_LOGIC;
			dbus_nxt : in STD_LOGIC;
			di : in STD_LOGIC_VECTOR (15 downto 0);
			dma : in STD_LOGIC;
			irdd : in STD_LOGIC_VECTOR (15 downto 0);
			mem_ce : in STD_LOGIC;
			pb : in STD_LOGIC_VECTOR (7 downto 0);
			read_nxt : in STD_LOGIC;
			rst : in STD_LOGIC;
			sw : in STD_LOGIC_VECTOR (7 downto 0);
			word_nxt : in STD_LOGIC;
			disp : out STD_LOGIC_VECTOR (15 downto 0);
			dma_req : out STD_LOGIC;
			do : out STD_LOGIC_VECTOR (15 downto 0);
			iadx : out STD_LOGIC_VECTOR (11 downto 0);
			insn : out STD_LOGIC_VECTOR (15 downto 0);
			int_req : out STD_LOGIC;
			iwrd : out STD_LOGIC_VECTOR (15 downto 0);
			iwrh : out STD_LOGIC;
			iwrl : out STD_LOGIC;
			ld_t : out STD_LOGIC;
			led : out STD_LOGIC_VECTOR (7 downto 0);
			rdy : out STD_LOGIC;
			ud_t : out STD_LOGIC;
			udlt_t : out STD_LOGIC;
			vadx : out STD_LOGIC_VECTOR (10 downto 0);
			vwrd : out STD_LOGIC_VECTOR (7 downto 0);
			vwren : out STD_LOGIC;
			zero_dma : out STD_LOGIC
			);
	end component;
	component mem2Kx16
		port (
			addr : in STD_LOGIC_VECTOR (11 downto 0);
			clk : in STD_LOGIC;
			di : in STD_LOGIC_VECTOR (15 downto 0);
			rst : in STD_LOGIC;
			weh : in STD_LOGIC;
			wel : in STD_LOGIC;
			do : out STD_LOGIC_VECTOR (15 downto 0)
			);
	end component;
	component vgachar
		port (
			adx : in STD_LOGIC_VECTOR (10 downto 0);
			clk25 : in STD_LOGIC;
			col : in STD_LOGIC_VECTOR (9 downto 0);
			hblk : in STD_LOGIC;
			row : in STD_LOGIC_VECTOR (9 downto 0);
			rst : in STD_LOGIC;
			vblk : in STD_LOGIC;
			wrd : in STD_LOGIC_VECTOR (7 downto 0);
			wren : in STD_LOGIC;
			b : out STD_LOGIC;
			g : out STD_LOGIC;
			i : out STD_LOGIC;
			r : out STD_LOGIC
			);
	end component;
	component xr16
		port (
			clk : in STD_LOGIC;
			dma_req : in STD_LOGIC;
			insn : in STD_LOGIC_VECTOR (15 downto 0);
			int_req : in STD_LOGIC;
			ld_t : in STD_LOGIC;
			rdy : in STD_LOGIC;
			rst : in STD_LOGIC;
			ud_t : in STD_LOGIC;
			udlt_t : in STD_LOGIC;
			zerodma : in STD_LOGIC;
			addr_nxt : out STD_LOGIC_VECTOR (15 downto 0);
			dbus_nxt : out STD_LOGIC;
			dma : out STD_LOGIC;
			mem_ce : out STD_LOGIC;
			read_nxt : out STD_LOGIC;
			word_nxt : out STD_LOGIC;
			d : inout STD_LOGIC_VECTOR (15 downto 0)
			);
	end component;
	
	---- Signal declarations used on the diagram ----
	
	signal clk: STD_LOGIC;
	
	signal ADDR : STD_LOGIC_VECTOR (11 downto 0);
	signal ADX_NXT : STD_LOGIC_VECTOR (15 downto 0);
	signal DBUS_NXT : STD_LOGIC;
	signal DDD : STD_LOGIC_VECTOR (15 downto 0);
	signal DDI : STD_LOGIC_VECTOR (15 downto 0);
	signal DDO : STD_LOGIC_VECTOR (15 downto 0);
	signal DI : STD_LOGIC_VECTOR (15 downto 0);
	signal DMA : STD_LOGIC;
	signal DMA_REQ : STD_LOGIC;
	signal DO : STD_LOGIC_VECTOR (15 downto 0);
	signal INSN : STD_LOGIC_VECTOR (15 downto 0);
	signal INT_REQ : STD_LOGIC;
	signal LD_T : STD_LOGIC;
	signal MEM_CE : STD_LOGIC;
	signal RDY : STD_LOGIC;
	signal READ_NXT : STD_LOGIC;
	signal UDLT_T : STD_LOGIC;
	signal UD_T : STD_LOGIC;
	signal VADX : STD_LOGIC_VECTOR (10 downto 0);
	signal VWRD : STD_LOGIC_VECTOR (7 downto 0);
	signal VWREN : STD_LOGIC;
	signal WEH : STD_LOGIC;
	signal WEL : STD_LOGIC;
	signal WORD_NXT : STD_LOGIC;
	signal ZERO_DMA : STD_LOGIC;
	
begin
	
	---- User defined VHDL code ----
	clk<= clk25;
	
	----- Statement0 ----
	ddi<= ddd;
	ddd<= ddo;
	
	----  Component instantiations  ----
	
	Uglue : Glue
	port map(
		adx_nxt => adx_nxt,
		clk => clk,
		dbus_nxt => dbus_nxt,
		di => ddi,
		disp => disp,
		dma => dma,
		dma_req => dma_req,
		do => ddo,
		iadx => addr,
		insn => insn,
		int_req => int_req,
		irdd => do,
		iwrd => di,
		iwrh => weh,
		iwrl => wel,
		ld_t => ld_t,
		led => led,
		mem_ce => mem_ce,
		pb => pb,
		rdy => rdy,
		read_nxt => read_nxt,
		rst => rst,
		sw => sw,
		ud_t => ud_t,
		udlt_t => udlt_t,
		vadx => vadx,
		vwrd => vwrd,
		vwren => vwren,
		word_nxt => word_nxt,
		zero_dma => zero_dma
		);
	
	Umem : mem2Kx16
	port map(
		addr => addr,
		clk => clk,
		di => di,
		do => do,
		rst => rst,
		weh => weh,
		wel => wel
		);
	
	Uvga : vgachar
	port map(
		adx => vadx,
		b => b,
		clk25 => clk,
		col => col,
		g => g,
		hblk => hblk,
		i => i,
		r => r,
		row => row,
		rst => rst,
		vblk => vblk,
		wrd => vwrd,
		wren => vwren
		);
	
	Uxr16 : xr16
	port map(
		addr_nxt => adx_nxt,
		clk => clk,
		d => ddd,
		dbus_nxt => dbus_nxt,
		dma => dma,
		dma_req => dma_req,
		insn => insn,
		int_req => int_req,
		ld_t => ld_t,
		mem_ce => mem_ce,
		rdy => rdy,
		read_nxt => read_nxt,
		rst => rst,
		ud_t => ud_t,
		udlt_t => udlt_t,
		word_nxt => word_nxt,
		zerodma => zero_dma
		);
	
	
end BHV;
--------------------------------------------------------------------------------
-- end of file
