--------------------------------------------------------------------------------
-- this file created automatically by hex2vhd.pl script.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
library UNISIM;
use UNISIM.vcomponents.all;

entity mem2Kx16 is
	port(
		clk:  in  STD_LOGIC;                      -- clock
		rst:  in  STD_LOGIC;                      -- reset
		addr: in  STD_LOGIC_VECTOR(10 downto 1);  -- address bus
		wel:  in  STD_LOGIC;                      -- write enable (low)
		weh:  in  STD_LOGIC;                      -- write enable (high)
		di:   in  STD_LOGIC_VECTOR(15 downto 0);  -- data input
		do:   out STD_LOGIC_VECTOR(15 downto 0)); -- data output
end mem2Kx16;

--------------------------------------------------------------------------------
architecture xilinx of mem2Kx16 is
	attribute INIT_00: string; attribute INIT_01: string;
	attribute INIT_02: string; attribute INIT_03: string;
	attribute INIT_04: string; attribute INIT_05: string;
	attribute INIT_06: string; attribute INIT_07: string;
	attribute INIT_08: string; attribute INIT_09: string;
	attribute INIT_0A: string; attribute INIT_0B: string;
	attribute INIT_0C: string; attribute INIT_0D: string;
	attribute INIT_0E: string; attribute INIT_0F: string;
	
	------------------------------------
	-- /* demo.c
	--  * demo: 8-bit and 16bit memory access
	--  */
	-- 
	-- #define TITLE (char*) 0x8000
	-- #define LED   (char*) 0x9000
	-- #define DISP  (short*)0xA000
	-- #define PB    (char*) 0xB000
	-- #define SW    (char*) 0xC000
	-- 
	-- char  d8 [] = {0x55, 0x42};
	-- short d16[] = {0x1704, 0x1601};
	-- 
	-- 
	-- void main()
	-- {	/* 8-bit test */
	-- 
	-- 	*LED= d8[0];
	-- 	*LED= d8[1];
	-- 
	-- 	*DISP= d16[0];
	-- 	*DISP= d16[1];
	-- 
	-- 	while(1) {}
	-- }
	-- 
	-- 
	attribute INIT_00 of U0: label is "A0B089DA59D089DA59D099D969D099D969D000161755A0C02DD0";
	attribute INIT_00 of U4: label is "F0FE00000C0000000A0000000900000008000001044200010EFF";
	------------------------------------
	
	signal en:                     STD_LOGIC_VECTOR(3 downto 0); -- BlockRAM enables
	signal do0l, do1l, do2l, do3l: STD_LOGIC_VECTOR(7 downto 0); -- BlockRAM data (low)
	signal do0h, do1h, do2h, do3h: STD_LOGIC_VECTOR(7 downto 0); -- BlockRAM data (high)
	alias  dil:                    STD_LOGIC_VECTOR(7 downto 0) is di(7 downto 0);
	alias  dih:                    STD_LOGIC_VECTOR(7 downto 0) is di(15 downto 8);
	signal addr2:                  STD_LOGIC_VECTOR(addr'range);
begin
	--	component ramb4_s8
	--	port(
	--		DI : in std_logic_vector(7 downto 0);
	--		EN : in std_ulogic;
	--		WE : in std_ulogic;
	--		RST : in std_ulogic;
	--		CLK : in std_ulogic;
	--		ADDR : in std_logic_vector(8 downto 0);
	--		DO : out std_logic_vector(7 downto 0));
	--	end component;
	
	-- *** edit U0..U7 generics to simulate ROM! ***
	
	-- big endian: even address, bits (15 downto 0)
	U0: RAMB4_S8 
	-- synopsys translate_off
	generic map(
	INIT_00 => x"000000000000A0B089DA59D089DA59D099D969D099D969D000161755A0C02DD0",
	INIT_01 => x"000000000000000000000000000000000000000000000000A02DB089DA29D099") 
	-- synopsys translate_on
	port map(	DI=> dih, EN=> en(0), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do0h);
	U1: RAMB4_S8 port map(	DI=> dih, EN=> en(1), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do1h);
	U2: RAMB4_S8 port map(	DI=> dih, EN=> en(2), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do2h);
	U3: RAMB4_S8 port map(	DI=> dih, EN=> en(3), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do3h);
	-- big endian: odd address, bits (7 downto 8)
	U4: RAMB4_S8 
	-- synopsys translate_off
	generic map(
	INIT_00 => x"000000000000F0FE00000C0000000A0000000900000008000001044200010EFF",
	INIT_01 => x"000000000000000000000000000000000000000000000000F0D4F600000A0A00") 
	-- synopsys translate_on
	port map(	DI=> dil, EN=> en(0), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do0l);
	U5: RAMB4_S8 port map(	DI=> dil, EN=> en(1), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do1l);
	U6: RAMB4_S8 port map(	DI=> dil, EN=> en(2), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do2l);
	U7: RAMB4_S8 port map(	DI=> dil, EN=> en(3), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do3l);
	
	process(clk, rst)
	begin
		if rst= '1' then
		elsif rising_edge(clk) then
			addr2<= addr;
		end if;
	end process;
	
	with addr2(10 downto 8) select do<=
	do0h & do0l when "000",
	do1h & do1l when "001",
	do2h & do2l when "010",
	do3h & do3l when others;
	
	with addr(10 downto 8) select en<=
	"0001" when "000",
	"0010" when "001",
	"0100" when "010",
	"1000" when others;
	
end architecture;

--------------------------------------------------------------------------------
-- end of file
