--------------------------------------------------------------------------------
-- Copyright (c) 2001 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2S Application Note: SoC with XR16 RISC CPU.
-- File:        Demo.vhd
-- Description: 2k x 16 memory with demo firmware.
-- History:     2001oct24, FB, Created.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- this file created automatically by hex2vhd.pl script.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
library UNISIM;
use UNISIM.vcomponents.all;

entity mem2Kx16 is
	port(
		clk:  in  STD_LOGIC;                      -- clock
		rst:  in  STD_LOGIC;                      -- reset
		addr: in  STD_LOGIC_VECTOR(11 downto 0);  -- address bus
		wel:  in  STD_LOGIC;                      -- write enable (low)
		weh:  in  STD_LOGIC;                      -- write enable (high)
		di:   in  STD_LOGIC_VECTOR(15 downto 0);  -- data input
		do:   out STD_LOGIC_VECTOR(15 downto 0)); -- data output
end mem2Kx16;

--------------------------------------------------------------------------------
architecture xilinx of mem2Kx16 is
	attribute INIT_00: string; attribute INIT_01: string;
	attribute INIT_02: string; attribute INIT_03: string;
	attribute INIT_04: string; attribute INIT_05: string;
	attribute INIT_06: string; attribute INIT_07: string;
	attribute INIT_08: string; attribute INIT_09: string;
	attribute INIT_0A: string; attribute INIT_0B: string;
	attribute INIT_0C: string; attribute INIT_0D: string;
	attribute INIT_0E: string; attribute INIT_0F: string;
	
	------------------------------------
	-- /* demo.c
	--  * demo: text output, push buttons, LEDs, 7-segments
	--  */
	-- 
	-- #define TITLE (char*) 0x8000
	-- #define LED   (char*) 0x9000
	-- #define DISP  (short*)0xA000
	-- #define PB    (char*) 0xB000
	-- #define SW    (char*) 0xC000
	-- 
	-- char msg[] = {"TE-XC2S App Note: SoC with XR16 RISC CPU"};
	-- 
	-- void main()
	-- {	char* src;
	-- 	char* dst;
	-- 
	-- 	src= msg;
	-- 	dst= TITLE;
	-- 	while(*src)
	-- 	{	*dst++ = *src++;
	-- 	}
	-- 
	-- 	while(1)
	-- 	{	*LED=  *SW;
	-- 		*DISP= *PB;
	-- 	}
	-- }
	-- 
	-- 
	attribute INIT_00 of U0: label is "000000000000000050205352365220747743533A744E704153432D54A0C02DD0";
	attribute INIT_01 of U0: label is "A02D5C5BB089DA69DB99D969DCB0B310699830682C082B09B02BD82CD08C8B2D";
	attribute INIT_00 of U4: label is "0000000000000000554343492031586869206F20656F20702032584500040EFF";
	attribute INIT_01 of U4: label is "F0D4D2D0F6000000000000000007F590C090008081C091B00600000800D2D0DC";
	------------------------------------
	
	signal en:                     STD_LOGIC_VECTOR(3 downto 0); -- BlockRAM enables
	signal do0l, do1l, do2l, do3l: STD_LOGIC_VECTOR(7 downto 0); -- BlockRAM data (low)
	signal do0h, do1h, do2h, do3h: STD_LOGIC_VECTOR(7 downto 0); -- BlockRAM data (high)
	alias  dil:                    STD_LOGIC_VECTOR(7 downto 0) is di(7 downto 0);
	alias  dih:                    STD_LOGIC_VECTOR(7 downto 0) is di(15 downto 8);
	signal addr2:                  STD_LOGIC_VECTOR(addr'range);
begin
	--	component ramb4_s8
	--	port(
	--		DI : in std_logic_vector(7 downto 0);
	--		EN : in std_ulogic;
	--		WE : in std_ulogic;
	--		RST : in std_ulogic;
	--		CLK : in std_ulogic;
	--		ADDR : in std_logic_vector(8 downto 0);
	--		DO : out std_logic_vector(7 downto 0));
	--	end component;
	
	-- *** edit U0..U7 generics to simulate ROM! ***
	
	-- big endian: even address, bits (15 downto 0)
	U0: RAMB4_S8 
	-- synopsys translate_off
	generic map(
	INIT_00 => x"000000000000000050205352365220747743533A744E704153432D54A0C02DD0",
	INIT_01 => x"A02D5C5BB089DA69DB99D969DCB0B310699830682C082B09B02BD82CD08C8B2D") 
	-- synopsys translate_on
	port map(	DI=> dih, EN=> en(0), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do0h);
	U1: RAMB4_S8 port map(	DI=> dih, EN=> en(1), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do1h);
	U2: RAMB4_S8 port map(	DI=> dih, EN=> en(2), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do2h);
	U3: RAMB4_S8 port map(	DI=> dih, EN=> en(3), WE=> weh, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do3h);
	-- big endian: odd address, bits (7 downto 8)
	U4: RAMB4_S8 
	-- synopsys translate_off
	generic map(
	INIT_00 => x"0000000000000000554343492031586869206F20656F20702032584500040EFF",
	INIT_01 => x"F0D4D2D0F6000000000000000007F590C090008081C091B00600000800D2D0DC") 
	-- synopsys translate_on
	port map(	DI=> dil, EN=> en(0), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do0l);
	U5: RAMB4_S8 port map(	DI=> dil, EN=> en(1), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do1l);
	U6: RAMB4_S8 port map(	DI=> dil, EN=> en(2), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do2l);
	U7: RAMB4_S8 port map(	DI=> dil, EN=> en(3), WE=> wel, RST=> rst, CLK=> clk, ADDR=> addr(9 downto 1), DO=> do3l);
	
	process(clk, rst)
	begin
		if rst= '1' then
		elsif rising_edge(clk) then
			addr2<= addr;
		end if;
	end process;
	
	with addr2(11 downto 10) select do<=
	do0h & do0l when "00",
	do1h & do1l when "01",
	do2h & do2l when "10",
	do3h & do3l when others;
	
	with addr(11 downto 10) select en<=
	"0001" when "00",
	"0010" when "01",
	"0100" when "10",
	"1000" when others;
	
end architecture;

--------------------------------------------------------------------------------
-- end of file
