--------------------------------------------------------------------------------
-- Copyright (c) 2001 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2S Application Note: SoC with XR16 RISC CPU.
-- File:        vgaChar.vhd
-- Description: VGA character display.
-- History:     2001oct02, FB, Created.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.NUMERIC_STD.all;

library UNISIM;
use UNISIM.vcomponents.all;

entity vgaCHAR is
	port (
		clk25: in STD_LOGIC;                       -- 25MHz pixel clock
		rst:   in STD_LOGIC;                       -- async reset
		-- character interface
		adx:   in STD_LOGIC_VECTOR (10 downto 0);  -- video RAM address
		wrd:   in STD_LOGIC_VECTOR (7 downto 0);   -- character data
		wren:  in STD_LOGIC;                       -- write enable
		-- VGA interface
		row:   in  STD_LOGIC_VECTOR(9 downto 0);   -- VGA row
		col:   in  STD_LOGIC_VECTOR(9 downto 0);   -- VGA col
		hblk:  in  STD_LOGIC;                      -- horizontal blanking
		vblk:  in  STD_LOGIC;                      -- vertical blanking
		r:     out STD_LOGIC;                      -- red
		g:     out STD_LOGIC;                      -- green
		b:     out STD_LOGIC;                      -- blue
		i:     out STD_LOGIC                       -- intensity
		);
end vgaCHAR;

--------------------------------------------------------------------------------
architecture xilinx of vgaCHAR is
	signal   trow:   UNSIGNED(5 downto 0);                  -- row of text (56..63, 0..31, 32..39)
	signal   tcol:   UNSIGNED(5 downto 0);                  -- col of text (0..63)
	signal   crow:   UNSIGNED(3 downto 0);                  -- row within character (9, 0..7, 8)
	signal   ccol:   UNSIGNED(3 downto 0);                  -- col within character (9, 0..7, 8)
	signal   cpel:   STD_LOGIC;                             -- char ROM pixel output
	signal   cc:     STD_LOGIC_VECTOR(7 downto 0);          -- current character (7bit)
	signal   rgbi:   STD_LOGIC_VECTOR(3 downto 0);          -- RGB + intensity
	constant rgb0:   STD_LOGIC_VECTOR(3 downto 0):= "1000"; -- background color
	constant rgb1:   STD_LOGIC_VECTOR(3 downto 0):= "1111"; -- foreground color
	--
	signal   one:    STD_LOGIC;                             -- logic '1'
	signal   zero:   STD_LOGIC;                             -- logic '0'
	
	--##############################################################################
	--	XST workaround, local signals taken from: crom_blk: block
	signal   cadx:   STD_LOGIC_VECTOR(8 downto 0);          -- adx in char ROM
	signal   cdi,cd: STD_LOGIC_VECTOR(7 downto 0);          -- char ROM mux output
	signal   cd1:    STD_LOGIC_VECTOR(7 downto 0);          -- char ROM #1 (0x00..0x3F) output
	signal   cd2:    STD_LOGIC_VECTOR(7 downto 0);          -- char ROM #2 (0x40..0x7F) output
	signal   crow2:  STD_LOGIC_VECTOR(2 downto 0);          -- ROM row index (upside down)
	signal   cc6:    STD_LOGIC;                             -- delayed cc(6)
	--
	signal   di:     STD_LOGIC_VECTOR(7 downto 0);          -- DI bus (unused)
	--------------------------------
	attribute INIT_00: string;
	attribute INIT_01: string;
	attribute INIT_02: string;
	attribute INIT_03: string;
	attribute INIT_04: string;
	attribute INIT_05: string;
	attribute INIT_06: string;
	attribute INIT_07: string;
	attribute INIT_08: string;
	attribute INIT_09: string;
	attribute INIT_0A: string;
	attribute INIT_0B: string;
	attribute INIT_0C: string;
	attribute INIT_0D: string;
	attribute INIT_0E: string;
	attribute INIT_0F: string;
	--------------------------------
	attribute INIT_00 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x03
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x02
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x01
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x00
	attribute INIT_01 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x07
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x06
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x05
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x04
	attribute INIT_02 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x0B
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x0A
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x09
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x08
	attribute INIT_03 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x0F
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x0E
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x0D
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x0C
	----------------
	attribute INIT_04 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x13
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x12
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x11
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x10
	attribute INIT_05 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x17
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x16
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x15
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x14
	attribute INIT_06 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x1B
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x1A
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x19
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x18
	attribute INIT_07 of U1: label is
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x1F
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x1E
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" & -- 0x1D
	"00" & "00" & "00" & "00" & "00" & "00" & "00" & "00" ; -- 0x1C
	attribute INIT_08 of U1: label is
	----------------
	"24" & -- "..#..#.." -- '#' 0x23
	"24" & -- "..#..#.."
	"FF" & -- "########"
	"24" & -- "..#..#.."
	"FF" & -- "########"
	"24" & -- "..#..#.."
	"24" & -- "..#..#.."
	"00" & -- "........"
	----------------
	"28" & -- "..#.#..." -- '"' 0x22
	"28" & -- "..#.#..."
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"18" & -- "...##..." -- '!' 0x21
	"18" & -- "...##..."
	"18" & -- "...##..."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" & -- "........"
	"10" & -- "...#...."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- ' ' 0x20
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" ; -- "........"
	attribute INIT_09 of U1: label is
	----------------
	"10" & -- "...#...." -- ''' 0x27
	"10" & -- "...#...."
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"60" & -- ".##....." -- '&' 0x26
	"90" & -- "#..#...."
	"90" & -- "#..#...."
	"60" & -- ".##....."
	"98" & -- "#..##..."
	"86" & -- "#....##."
	"79" & -- ".####..#"
	"00" & -- "........"
	----------------
	"C0" & -- "##......" -- '%' 0x25
	"C1" & -- "##.....#"
	"06" & -- ".....##."
	"18" & -- "...##..."
	"60" & -- ".##....."
	"83" & -- "#.....##"
	"03" & -- "......##"
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- '$' 0x24
	"7E" & -- ".######."
	"90" & -- "#..#...."
	"7E" & -- ".######."
	"11" & -- "...#...#"
	"7E" & -- ".######."
	"10" & -- "...#...."
	"00" ; -- "........"
	attribute INIT_0A of U1: label is
	----------------
	"00" & -- "........" -- '+' 0x2B
	"10" & -- "...#...."
	"10" & -- "...#...."
	"7C" & -- ".#####.."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '*' 0x2A
	"10" & -- "...#...."
	"10" & -- "...#...."
	"D6" & -- "##.#.##."
	"28" & -- "..#.#..."
	"44" & -- ".#...#.."
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- ')' 0x29
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"10" & -- "...#...."
	"00" & -- "........"
	----------------
	"08" & -- "....#..." -- '(' 0x28
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"08" & -- "....#..."
	"00" ; -- "........"
	attribute INIT_0B of U1: label is
	----------------
	"02" & -- "......#." -- '/' 0x2F
	"04" & -- ".....#.."
	"08" & -- "....#..."
	"10" & -- "...#...."
	"20" & -- "..#....."
	"40" & -- ".#......"
	"80" & -- "#......."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '.' 0x2E
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"18" & -- "...##..."
	"18" & -- "...##..."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '-' 0x2D
	"00" & -- "........"
	"00" & -- "........"
	"7C" & -- ".#####.."
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- ',' 0x2C
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"18" & -- "...##..."
	"18" & -- "...##..."
	"10" ; -- "...#...."
	attribute INIT_0C of U1: label is
	----------------
	"7E" & -- ".######." -- '3' 0x33
	"81" & -- "#......#"
	"01" & -- ".......#"
	"7E" & -- ".######."
	"01" & -- ".......#"
	"01" & -- ".......#"
	"FE" & -- "#######."
	"00" & -- "........"
	----------------
	"7E" & -- ".######." -- '2' 0x32
	"81" & -- "#......#"
	"01" & -- ".......#"
	"7E" & -- ".######."
	"80" & -- "#......."
	"80" & -- "#......."
	"FF" & -- "########"
	"00" & -- "........"
	----------------
	"08" & -- "....#..." -- '1' 0x31
	"18" & -- "...##..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"1C" & -- "...###.."
	"00" & -- "........"
	----------------
	"7E" & -- ".######." -- '0' 0x30
	"81" & -- "#......#"
	"89" & -- "#...#..#"
	"91" & -- "#..#...#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" ; -- "........"
	attribute INIT_0D of U1: label is
	----------------
	"FF" & -- "########" -- '7' 0x37
	"02" & -- "......#."
	"04" & -- ".....#.."
	"08" & -- "....#..."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- '6' 0x36
	"20" & -- "..#....."
	"40" & -- ".#......"
	"FE" & -- "#######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"FF" & -- "########" -- '5' 0x35
	"80" & -- "#......."
	"80" & -- "#......."
	"FE" & -- "#######."
	"01" & -- ".......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"18" & -- "...##..." -- '4' 0x34
	"28" & -- "..#.#..."
	"48" & -- ".#..#..."
	"FE" & -- "#######."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"1C" & -- "...###.."
	"00" ; -- "........"
	attribute INIT_0E of U1: label is
	----------------
	"00" & -- "........" -- ';' 0x3B
	"00" & -- "........"
	"18" & -- "...##..."
	"18" & -- "...##..."
	"00" & -- "........"
	"18" & -- "...##..."
	"18" & -- "...##..."
	"10" & -- "...#...."
	----------------
	"00" & -- "........" -- ':' 0x3A
	"00" & -- "........"
	"18" & -- "...##..."
	"18" & -- "...##..."
	"00" & -- "........"
	"18" & -- "...##..."
	"18" & -- "...##..."
	"00" & -- "........"
	----------------
	"7E" & -- ".######." -- '9' 0x39
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"02" & -- "......#."
	"04" & -- ".....#.."
	"08" & -- "....#..."
	"00" & -- "........"
	----------------
	"7E" & -- ".######." -- '8' 0x38
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" ; -- "........"
	attribute INIT_0F of U1: label is
	----------------
	"7E" & -- ".######." -- '?' 0x3F
	"81" & -- "#......#"
	"06" & -- ".....##."
	"08" & -- "....#..."
	"10" & -- "...#...."
	"00" & -- "........"
	"10" & -- "...#...."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '>' 0x3E
	"60" & -- ".##....."
	"18" & -- "...##..."
	"06" & -- ".....##."
	"18" & -- "...##..."
	"60" & -- ".##....."
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '=' 0x3D
	"00" & -- "........"
	"7E" & -- ".######."
	"00" & -- "........"
	"7E" & -- ".######."
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '< 0x3C
	"06" & -- ".....##."
	"18" & -- "...##..."
	"60" & -- ".##....."
	"18" & -- "..##...."
	"06" & -- ".....##."
	"00" & -- "........"
	"00" ; -- "........"
	attribute INIT_00 of U2: label is
	----------------
	"7E" & -- ".######." -- 'C' 0x43
	"81" & -- "#......#"
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"FE" & -- "#######." -- 'B' 0x42
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"00" & -- "........"
	----------------
	"18" & -- "...##..." -- 'A' 0x41
	"24" & -- "..#..#.."
	"42" & -- ".#....#."
	"FF" & -- "########"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"00" & -- "........"
	----------------
	"7E" & -- ".######." -- '@' 0x40
	"81" & -- "#......#"
	"9F" & -- "#..#####"
	"A1" & -- "#.#....#"
	"9F" & -- "#..#####"
	"80" & -- "#......."
	"82" & -- "#.....#."
	"7C" ; -- ".#####.."
	attribute INIT_01 of U2: label is
	----------------
	"7E" & -- ".######." -- 'G' 0x47
	"81" & -- "#......#"
	"80" & -- "#......."
	"8F" & -- "#...####"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"00" & -- "........"
	----------------
	"FF" & -- "########" -- 'F' 0x46
	"80" & -- "#......."
	"80" & -- "#......."
	"FC" & -- "######.."
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"00" & -- "........"
	----------------
	"FF" & -- "########" -- 'E' 0x45
	"80" & -- "#......."
	"80" & -- "#......."
	"FC" & -- "######.."
	"80" & -- "#......."
	"80" & -- "#......."
	"FF" & -- "########"
	"00" & -- "........"
	----------------
	"FE" & -- "#######." -- 'D' 0x44
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"00" ; -- "........"
	attribute INIT_02 of U2: label is
	----------------
	"81" & -- "#......#" -- 'K' 0x4B
	"86" & -- "#....##."
	"98" & -- "#..##..."
	"E0" & -- "###....."
	"98" & -- "#..##..."
	"86" & -- "#....##."
	"81" & -- "#......#"
	"00" & -- "........"
	----------------
	"01" & -- ".......#" -- 'J' 0x4A
	"01" & -- ".......#"
	"01" & -- ".......#"
	"01" & -- ".......#"
	"01" & -- ".......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"38" & -- "..###..." -- 'I' 0x49
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"38" & -- "..###..."
	"00" & -- "........"
	----------------
	"81" & -- "#......#" -- 'H' 0x48
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FF" & -- "########"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"00" ; -- "........"
	attribute INIT_03 of U2: label is
	----------------
	"7E" & -- ".######." -- 'O' 0x4F
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"C1" & -- "##.....#" -- 'N' 0x4E
	"A1" & -- "#.#....#"
	"91" & -- "#..#...#"
	"89" & -- "#...#..#"
	"85" & -- "#....#.#"
	"83" & -- "#.....##"
	"81" & -- "#......#"
	"00" & -- "........"
	----------------
	"81" & -- "#......#" -- 'M' 0x4D
	"C3" & -- "##....##"
	"A5" & -- "#.#..#.#"
	"99" & -- "#..##..#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"00" & -- "........"
	----------------
	"80" & -- "#......." -- 'L' 0x4C
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"FF" & -- "########"
	"00" ; -- "........"
	attribute INIT_04 of U2: label is
	----------------
	"7E" & -- ".######." -- 'S' 0x53
	"81" & -- "#......#"
	"80" & -- "#......."
	"7E" & -- ".######."
	"01" & -- ".......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"FE" & -- "#######." -- 'R' 0x52
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"00" & -- "........"
	----------------
	"7E" & -- ".######." -- 'Q' 0x51
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"85" & -- "#....#.#"
	"82" & -- "#.....#."
	"7D" & -- ".#####.#"
	"00" & -- "........"
	----------------
	"FE" & -- "#######." -- 'P' 0x50
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"00" ; -- "........"
	attribute INIT_05 of U2: label is
	----------------
	"81" & -- "#......#" -- 'W' 0x57
	"81" & -- "#......#"
	"81" & -- "#......#"
	"99" & -- "#..##..#"
	"A5" & -- "#.#..#.#"
	"42" & -- ".#....#."
	"42" & -- ".#....#."
	"00" & -- "........"
	----------------
	"81" & -- "#......#" -- 'V' 0x56
	"81" & -- "#......#"
	"42" & -- ".#....#."
	"42" & -- ".#....#."
	"24" & -- "..#..#.."
	"24" & -- "..#..#.."
	"18" & -- "...##..."
	"00" & -- "........"
	----------------
	"81" & -- "#......#" -- 'U' 0x55
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"FF" & -- "########" -- 'T' 0x54
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" ; -- "........"
	attribute INIT_06 of U2: label is
	----------------
	"1C" & -- "...###.." -- '[' 0x5B
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"1C" & -- "...###.."
	"00" & -- "........"
	----------------
	"FF" & -- "########" -- 'Z' 0x5A
	"02" & -- "......#."
	"04" & -- ".....#.."
	"18" & -- "...##..."
	"20" & -- "..#....."
	"40" & -- ".#......"
	"FF" & -- "########"
	"00" & -- "........"
	----------------
	"81" & -- "#......#" -- 'Y' 0x59
	"81" & -- "#......#"
	"42" & -- ".#....#."
	"3C" & -- "..####.."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" & -- "........"
	----------------
	"81" & -- "#......#" -- 'X' 0x58
	"42" & -- ".#....#."
	"24" & -- "..#..#.."
	"18" & -- "...##..."
	"24" & -- "..#..#.."
	"42" & -- ".#....#."
	"81" & -- "#......#"
	"00" ; -- "........"
	attribute INIT_07 of U2: label is
	----------------
	"00" & -- "........" -- '_ 0x5F
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"FF" & -- "########"
	----------------
	"10" & -- "...#...." -- '^' 0x5E
	"28" & -- "..#.#..."
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"38" & -- "..###..." -- ']' 0x5D
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"08" & -- "....#..."
	"38" & -- "..###..."
	"00" & -- "........"
	----------------
	"80" & -- "#......." -- '\' 0x5C
	"40" & -- ".#......"
	"20" & -- "..#....."
	"10" & -- "...#...."
	"08" & -- "....#..."
	"04" & -- ".....#.."
	"02" & -- "......#."
	"00" ; -- "........"
	attribute INIT_08 of U2: label is
	----------------
	"00" & -- "........" -- 'c' 0x63
	"00" & -- "........"
	"7E" & -- ".######."
	"81" & -- "#......#"
	"80" & -- "#......."
	"80" & -- "#......."
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"80" & -- "#......." -- 'b' 0x62
	"80" & -- "#......."
	"FE" & -- "#######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'a' 0x61
	"00" & -- "........"
	"7E" & -- ".######."
	"01" & -- ".......#"
	"7F" & -- ".#######"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- '`' 0x60
	"08" & -- "....#..."
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" ; -- "........"
	attribute INIT_09 of U2: label is
	----------------
	"00" & -- "........" -- 'g' 0x67
	"00" & -- "........"
	"7F" & -- ".#######"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"01" & -- ".......#"
	"7E" & -- ".######."
	----------------
	"0C" & -- "....##.." -- 'f' 0x66
	"10" & -- "...#...."
	"7E" & -- ".######."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'e' 0x65
	"00" & -- "........"
	"7E" & -- ".######."
	"81" & -- "#......#"
	"FF" & -- "########"
	"80" & -- "#......."
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"01" & -- ".......#" -- 'd' 0x64
	"01" & -- ".......#"
	"7F" & -- ".#######"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"00" ; -- "........"
	attribute INIT_0A of U2: label is
	----------------
	"80" & -- "#......." -- 'k' 0x6B
	"80" & -- "#......."
	"83" & -- "#.....##"
	"8C" & -- "#...##.."
	"F0" & -- "####...."
	"8C" & -- "#...##.."
	"83" & -- "#.....##"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'j' 0x6A
	"00" & -- "........"
	"10" & -- "...#...."
	"00" & -- "........"
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"60" & -- ".##....."
	----------------
	"00" & -- "........" -- 'i' 0x69
	"00" & -- "........"
	"10" & -- "...#...."
	"00" & -- "........"
	"10" & -- "...#...."
	"10" & -- "...#...."
	"0C" & -- "....##.."
	"00" & -- "........"
	----------------
	"80" & -- "#......." -- 'h' 0x68
	"80" & -- "#......."
	"FE" & -- "#######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"00" ; -- "........"
	attribute INIT_0B of U2: label is
	----------------
	"00" & -- "........" -- 'o' 0x6F
	"00" & -- "........"
	"7E" & -- ".######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'n' 0x6E
	"00" & -- "........"
	"FC" & -- "######.."
	"82" & -- "#.....#."
	"82" & -- "#.....#."
	"82" & -- "#.....#."
	"83" & -- "#.....##"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'm' 0x6D
	"00" & -- "........"
	"EC" & -- "###.##.."
	"92" & -- "#..#..#."
	"92" & -- "#..#..#."
	"92" & -- "#..#..#."
	"93" & -- "#..#..##"
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- 'l' 0x6C
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"0C" & -- "....##.."
	"00" ; -- "........"
	attribute INIT_0C of U2: label is
	----------------
	"00" & -- "........" -- 's' 0x73
	"00" & -- "........"
	"7E" & -- ".######."
	"80" & -- "#......."
	"7E" & -- ".######."
	"01" & -- ".......#"
	"7E" & -- ".######."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'r' 0x72
	"00" & -- "........"
	"BE" & -- "#.#####."
	"C1" & -- "##.....#"
	"80" & -- "#......."
	"80" & -- "#......."
	"80" & -- "#......."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'q' 0x71
	"00" & -- "........"
	"7F" & -- ".#######"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"01" & -- ".......#"
	"01" & -- ".......#"
	----------------
	"00" & -- "........" -- 'p' 0x70
	"00" & -- "........"
	"FE" & -- "#######."
	"81" & -- "#......#"
	"81" & -- "#......#"
	"FE" & -- "#######."
	"80" & -- "#......."
	"80" ; -- "#......."
	attribute INIT_0D of U2: label is
	----------------
	"00" & -- "........" -- 'w' 0x77
	"00" & -- "........"
	"91" & -- "#..#...#"
	"91" & -- "#..#...#"
	"91" & -- "#..#...#"
	"91" & -- "#..#...#"
	"6E" & -- ".##.###."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'v' 0x76
	"00" & -- "........"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"42" & -- ".#....#."
	"42" & -- ".#....#."
	"3C" & -- "..####.."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'u' 0x75
	"00" & -- "........"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- 't' 0x74
	"10" & -- "...#...."
	"7E" & -- ".######."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"0C" & -- "....##.."
	"00" ; -- "........"
	attribute INIT_0E of U2: label is
	----------------
	"08" & -- "....#..." -- '{' 0x7B
	"10" & -- "...#...."
	"10" & -- "...#...."
	"20" & -- "..#....."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"08" & -- "....#..."
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'z' 0x7A
	"00" & -- "........"
	"FF" & -- "########"
	"04" & -- ".....#.."
	"18" & -- "...##..."
	"20" & -- "..#....."
	"FF" & -- "########"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- 'y' 0x79
	"00" & -- "........"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"81" & -- "#......#"
	"7F" & -- ".#######"
	"01" & -- ".......#"
	"7E" & -- ".######."
	----------------
	"00" & -- "........" -- 'x' 0x78
	"00" & -- "........"
	"C3" & -- "##....##"
	"24" & -- "..#..#.."
	"18" & -- "...##..."
	"24" & -- "..#..#.."
	"C3" & -- "##....##"
	"00" ; -- "........"
	attribute INIT_0F of U2: label is
	----------------
	"00" & -- "........" -- *** 0x7F
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"00" & -- "........" -- '~' 0x7E
	"00" & -- "........"
	"02" & -- "......#."
	"3C" & -- "..####.."
	"40" & -- ".#......"
	"00" & -- "........"
	"00" & -- "........"
	"00" & -- "........"
	----------------
	"20" & -- "..#....." -- '}' 0x7D
	"10" & -- "...#...."
	"10" & -- "...#...."
	"08" & -- "....#..."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"20" & -- "..#....."
	"00" & -- "........"
	----------------
	"10" & -- "...#...." -- '|' 0x7C
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"10" & -- "...#...."
	"00" ; -- "........"
	--------------------------------
	--##############################################################################
	--XST workaround, local signals taken from: vram_blk: block
	signal   en1, en2, en3, en4: STD_LOGIC;                     -- video RAM write enable
	signal   cc1, cc2, cc3, cc4: STD_LOGIC_VECTOR(7 downto 0);  -- video RAM output
	signal   tadx:               STD_LOGIC_VECTOR(10 downto 0); -- adx in video RAM
	--
	signal   dib:      STD_LOGIC_VECTOR(7 downto 0);            -- DIB bus (unused)
	--##############################################################################
begin
	one<= '1';
	zero<= '0';
	
	rgbi<= rgb1 when cpel= '1' and crow(3)= '0' and trow(5)= '0' else rgb0;
	r<= rgbi(3);
	g<= rgbi(2);
	b<= rgbi(1);
	i<= rgbi(0);
	
	------------------------------------
	-- row/ col generation from sync signals
	rowcol_blk: block
		signal hblk1:  STD_LOGIC;
		signal hblken: STD_LOGIC;
		begin
		
		process(rst, clk25)
		begin
			if rst= '1' then
				hblk1<= '0';
			elsif rising_edge(clk25) then
				hblk1<= hblk;
			end if;
		end process;
		hblken<= hblk and not(hblk1);
		
		process(rst, clk25)
		begin
			if rst= '1' then
				trow<= TO_UNSIGNED(0, trow'length);
				crow<= TO_UNSIGNED(0, crow'length);
				tcol<= TO_UNSIGNED(0, tcol'length);
				ccol<= TO_UNSIGNED(0, ccol'length);
			elsif rising_edge(clk25) then
				if vblk= '1' then
					trow<= TO_UNSIGNED(56, trow'length);
					crow<= TO_UNSIGNED(0,  crow'length);
				elsif hblken= '1' then
					crow<= crow + 1;
					if crow= TO_UNSIGNED(9, crow'length) then
						trow<= trow + 1;
						crow<= TO_UNSIGNED(0, crow'length);
					end if;
				end if;
				
				if hblk= '1' then
					tcol<= TO_UNSIGNED(0, tcol'length);
					ccol<= TO_UNSIGNED(9, ccol'length);
				else
					ccol<= ccol + 1;
					if ccol= TO_UNSIGNED(9, ccol'length) then
						cd<= cdi;
						tcol<= tcol + 1;
						ccol<= TO_UNSIGNED(0, ccol'length);
					end if;
				end if;
			end if;
		end process;
	end block;
	
	------------------------------------
	-- character ROM
	--	crom_blk: block
	--		begin
	di  <= (others=> zero);
	
	cadx<= cc(5 downto 0) & STD_LOGIC_VECTOR(crow2);
	with crow select crow2<= 
	"111" when "0000",
	"110" when "0001",
	"101" when "0010",
	"100" when "0011",
	"011" when "0100",
	"010" when "0101",
	"001" when "0110",
	"000" when others;
	
	
	U1: RAMB4_S8 port map(DI=> di, EN=> one, WE=> zero, RST=> rst, CLK=> clk25, ADDR=>cadx, DO=> cd1);
	U2: RAMB4_S8 
	--                      ccccccccccccccccbbbbbbbbbbbbbbbbaaaaaaaaaaaaaaaa@@@@@@@@@@@@@@@@
	--	generic map(INIT_00=> x"0000000000000000818181818181818100000000000000008181818181818181")
	port map(DI=> di, EN=> one, WE=> zero, RST=> rst, CLK=> clk25, ADDR=>cadx, DO=> cd2);
	
	process(clk25, rst)
	begin
		if rst= '1' then
			cc6<= '0';
		elsif rising_edge(clk25) then
			cc6<= cc(6);
		end if;
	end process;
	
	cdi<= cd1 when cc6= '0' else cd2;
	
	with ccol select cpel<=
	'0'   when "1001",
	cd(7) when "0000",
	cd(6) when "0001",
	cd(5) when "0010",
	cd(4) when "0011",
	cd(3) when "0100",
	cd(2) when "0101",
	cd(1) when "0110",
	cd(0) when "0111",
	'0'   when others;
	
	--	end block;
	
	------------------------------------
	-- video RAM, 32 lines x 64 chars
	--	vram_blk: block
	--		begin
	en1<= '1' when adx(10 downto 9)= "00" else '0';
	en2<= '1' when adx(10 downto 9)= "01" else '0';
	en3<= '1' when adx(10 downto 9)= "10" else '0';
	en4<= '1' when adx(10 downto 9)= "11" else '0';
	
	tadx<= STD_LOGIC_VECTOR(trow(4 downto 0)) & STD_LOGIC_VECTOR(tcol);
	
	U3: RAMB4_S8_S8 
	--generic map(INIT_00=> x"4342414043424140434241404342414043424140434241404342414043424140",
	--INIT_01=> x"4342414043424140434241404342414043424140434241404342414043424140")
	port map(
		DIA=> wrd, ENA=> en1, WEA=> wren, RSTA=> zero, CLKA=> clk25, ADDRA=> adx(8 downto 0),  DOA=> open,
		DIB=> dib, ENB=> one, WEB=> zero, RSTB=> zero, CLKB=> clk25, ADDRB=> tadx(8 downto 0), DOB=> cc1);
	U4: RAMB4_S8_S8 port map(
		DIA=> wrd, ENA=> en2, WEA=> wren, RSTA=> zero, CLKA=> clk25, ADDRA=> adx(8 downto 0),  DOA=> open,
		DIB=> dib, ENB=> one, WEB=> zero, RSTB=> zero, CLKB=> clk25, ADDRB=> tadx(8 downto 0), DOB=> cc2);
	U5: RAMB4_S8_S8 port map(
		DIA=> wrd, ENA=> en3, WEA=> wren, RSTA=> zero, CLKA=> clk25, ADDRA=> adx(8 downto 0),  DOA=> open,
		DIB=> dib, ENB=> one, WEB=> zero, RSTB=> zero, CLKB=> clk25, ADDRB=> tadx(8 downto 0), DOB=> cc3);
	U6: RAMB4_S8_S8 port map(
		DIA=> wrd, ENA=> en4, WEA=> wren, RSTA=> zero, CLKA=> clk25, ADDRA=> adx(8 downto 0),  DOA=> open,
		DIB=> dib, ENB=> one, WEB=> zero, RSTB=> zero, CLKB=> clk25, ADDRB=> tadx(8 downto 0), DOB=> cc4);
	
	with tadx(10 downto 9) select cc<=
	cc1 when "00",
	cc2 when "01",
	cc3 when "10",
	cc4 when others;
	
	dib<= (others=> '-');
	
	--	end block;
end xilinx;
--------------------------------------------------------------------------------
-- end of file
